# test_valuerep.py
"""Test suite for valuerep.py"""
# Copyright (c) 2008 Darcy Mason
# This file is part of pydicom, released under a modified MIT license.
#    See the file license.txt included with this distribution, also
#    available at http://pydicom.googlecode.com

import unittest
from dicom.valuerep import PersonName, PersonNameUnicode

default_encoding = 'iso8859'

class PersonNametests(unittest.TestCase):
    def testLastFirst(self):
        """PN: Simple Family-name^Given-name works..............................."""
        pn = PersonName("Family^Given")
        expected = "Family"
        got = pn.family_name
        self.assertEqual(got, expected, "PN: expected '%s', got '%s' for family name" % (expected, got))
        
        expected = 'Given'
        got = pn.given_name
        self.assertEqual(got, expected, "PN: expected '%s', got '%s' for given name" % (expected, got))
        
        expected = ''
        got = pn.name_suffix
        self.assertEqual(got, expected, "PN: expected '%s', got '%s' for name_suffix" % (expected, got))
        
        expected = ''
        got = pn.phonetic
        self.assertEqual(got, expected, "PN: expected '%s', got '%s' for phonetic component" % (expected, got))
        
    def testThreeComponent(self):
        """PN: 3component (single-byte, ideographic, phonetic characters) works.."""
        # Example name from PS3.5-2008 section I.2 p. 108
        pn = PersonName("""Hong^Gildong=\033$)C\373\363^\033$)C\321\316\324\327=\033$)C\310\253^\033$)C\261\346\265\277""")
        expected = ("Hong", "Gildong")
        got = (pn.family_name, pn.given_name)
        self.assertEqual(got, expected, "PN: Expected single_byte name '%s', got '%s'" % (expected, got))
    def testFormatting(self):
        """PN: Formatting works.................................................."""
        pn = PersonName("Family^Given")
        expected = "Family, Given"
        got = pn.family_comma_given()
        self.assertEqual(got, expected, "PN: expected '%s', got '%s' for formatted Family, Given" % (expected, got))
    def testUnicodeKr(self):
        """PN: 3component in unicode works (Korean).............................."""
        # Example name from PS3.5-2008 section I.2 p. 101
        from sys import version_info
        if version_info >= (2,4):
            pn = PersonNameUnicode(
               """Hong^Gildong=\033$)C\373\363^\033$)C\321\316\324\327=\033$)C\310\253^\033$)C\261\346\265\277""",
               [default_encoding,'euc_kr'])
            expected = ("Hong", "Gildong")
            got = (pn.family_name, pn.given_name)
            self.assertEqual(got, expected, "PN: Expected single_byte name '%s', got '%s'" % (expected, got))
    def testUnicodeJp(self):
        """PN: 3component in unicode works (Japanese)............................"""
        # Example name from PS3.5-2008 section H  p. 98
        from sys import version_info
        if version_info >= (2,4):
            pn = PersonNameUnicode(
               """Yamada^Tarou=\033$B;3ED\033(B^\033$BB@O:\033(B=\033$B$d$^$@\033(B^\033$B$?$m$&\033(B""",
               [default_encoding,'iso2022_jp'])
            expected = ("Yamada", "Tarou")
            got = (pn.family_name, pn.given_name)
            self.assertEqual(got, expected, "PN: Expected single_byte name '%s', got '%s'" % (expected, got))
        
if __name__ == "__main__":
    unittest.main()



# -*- coding: latin_1 -*-
# test_charset.py
"""unittest cases for dicom.charset module"""
# Copyright (c) 2008 Darcy Mason
# This file is part of pydicom, released under a modified MIT license.
#    See the file license.txt included with this distribution, also
#    available at http://pydicom.googlecode.com

import unittest
import dicom
import os.path

from pkg_resources import Requirement, resource_filename
testcharset_dir = resource_filename(Requirement.parse("pydicom"),"dicom/testcharsetfiles")

latin1_file = os.path.join(testcharset_dir, "chrFren.dcm")
jp_file = os.path.join(testcharset_dir, "chrH31.dcm")
multiPN_file = os.path.join(testcharset_dir, "chrFrenMulti.dcm")

test_dir = resource_filename(Requirement.parse("pydicom"),"dicom/testfiles")
normal_file = os.path.join(test_dir, "CT_small.dcm")


class charsetTests(unittest.TestCase):
    def testLatin1(self):
        """charset: can read and decode latin_1 file........................"""
        ds = dicom.read_file(latin1_file)
        ds.decode()
        # Make sure don't get unicode encode error on converting to string
        expected = u"Buc^Jérôme"
        got = ds.PatientName
        self.assertEqual(expected, got, "Expected %r, got %r" % (expected, got))
    def testStandardFile(self):
        """charset: can read and decode standard file without special char.."""
        ds = dicom.read_file(normal_file)
        ds.decode()
    def testMultiPN(self):
        """charset: can decode file with multi-valued data elements........."""
        ds = dicom.read_file(multiPN_file)
        ds.decode()

if __name__ == "__main__":
    # This is called if run alone, but not if loaded through run_tests.py
    # If not run from the directory where the sample images are, then need to switch there
    import sys
    import os
    import os.path
    dir_name = os.path.dirname(sys.argv[0])
    save_dir = os.getcwd()
    if dir_name:
        os.chdir(dir_name)
    os.chdir("../testfiles")
    unittest.main()
    os.chdir(save_dir)


# test_rawread.py
"""unittest tests for dicom.filereader module -- simple raw data elements"""
# Copyright (c) 2010 Darcy Mason
# This file is part of pydicom, relased under an MIT license.
#    See the file license.txt included with this distribution, also
#    available at http://pydicom.googlecode.com

from cStringIO import StringIO
import unittest
from dicom.filereader import data_element_generator
from dicom.values import convert_value
from dicom.sequence import Sequence

def hex2str(hexstr):
    """Return a bytestring rep of a string of hex rep of bytes separated by spaces"""
    return "".join((chr(int(x,16)) for x in hexstr.split()))

class RawReaderExplVRTests(unittest.TestCase):
    # See comments in data_element_generator -- summary of DICOM data element formats
    # Here we are trying to test all those variations

    def testExplVRLittleEndianLongLength(self):
        """Raw read: Explicit VR Little Endian long length......................"""
        # (0002,0001) OB 2-byte-reserved 4-byte-length, value 0x00 0x01
        infile = StringIO(hex2str("02 00 01 00 4f 42 00 00 02 00 00 00 00 01"))
        expected = ((2,1), 'OB', 2, '\00\01', 0xc, False, True)
        de_gen = data_element_generator(infile, is_implicit_VR=False, is_little_endian=True)
        got = de_gen.next()
        msg_loc = "in read of Explicit VR='OB' data element (long length format)"
        self.assertEqual(got, expected, "Expected: %r, got %r in %s" % (expected, got, msg_loc))
        # (0002,0002) OB 2-byte-reserved 4-byte-length, value 0x00 0x01
    def testExplVRLittleEndianShortLength(self):
        """Raw read: Explicit VR Little Endian short length....................."""
        # (0008,212a) IS 2-byte-length, value '1 '
        infile = StringIO(hex2str("08 00 2a 21 49 53 02 00 31 20"))
        expected = ((8,0x212a), 'IS', 2, '1 ', 0x8, False, True)
        de_gen = data_element_generator(infile, is_implicit_VR=False, is_little_endian=True)
        got = de_gen.next()
        msg_loc = "in read of Explicit VR='IS' data element (short length format)"
        self.assertEqual(got, expected, "Expected: %r, got %r in %s" % (expected, got, msg_loc))
    def testExplVRLittleEndianUndefLength(self):
        """Raw read: Expl VR Little Endian with undefined length................"""
        # (7fe0,0010), OB, 2-byte reserved, 4-byte-length (UNDEFINED)
        bytes1 = "e0 7f 10 00 4f 42 00 00 ff ff ff ff"
        bytes2 = " 41 42 43 44 45 46 47 48 49 4a"  # 'content'
        bytes3 = " fe ff dd e0 00 00 00 00"          # Sequence Delimiter
        bytes = bytes1 + bytes2 + bytes3
        infile = StringIO(hex2str(bytes))
        expected = ((0x7fe0,0x10), 'OB', 0xffffffffL, 'ABCDEFGHIJ', 0xc, False, True)
        de_gen = data_element_generator(infile, is_implicit_VR=False, is_little_endian=True)
        got = de_gen.next()
        msg_loc = "in read of undefined length Explicit VR ='OB' short value)"
        self.assertEqual(got, expected, "Expected: %r, got %r in %s" % (expected, got, msg_loc))

        # Test again such that delimiter crosses default 128-byte read "chunks", etc
        for multiplier in (116, 117, 118, 120):
            multiplier = 116
            bytes2b = bytes2 + " 00"*multiplier
            bytes = bytes1 + bytes2b + bytes3
            infile = StringIO(hex2str(bytes))
            expected = len('ABCDEFGHIJ'+'\0'*multiplier)
            de_gen = data_element_generator(infile, is_implicit_VR=False, is_little_endian=True)
            got = de_gen.next()
            got_len = len(got.value)
            msg_loc = "in read of undefined length Explicit VR ='OB' with 'multiplier' %d" % multiplier
            self.assertEqual(expected, got_len, "Expected value length %d, got %d in %s" % (expected, got_len, msg_loc))
            msg = "Unexpected value start with multiplier %d on Expl VR undefined length" % multiplier
            self.assert_(got.value.startswith('ABCDEFGHIJ\0'), msg)
            
class RawReaderImplVRTests(unittest.TestCase):
    # See comments in data_element_generator -- summary of DICOM data element formats
    # Here we are trying to test all those variations

    def testImplVRLittleEndian(self):
        """Raw read: Implicit VR Little Endian.................................."""
        # (0008,212a) {IS} 4-byte-length, value '1 '
        infile = StringIO(hex2str("08 00 2a 21 02 00 00 00 31 20"))
        expected = ((8,0x212a), None, 2, '1 ', 0x8, True, True)
        de_gen = data_element_generator(infile, is_implicit_VR=True, is_little_endian=True)
        got = de_gen.next()
        msg_loc = "in read of Implicit VR='IS' data element (short length format)"
        self.assertEqual(got, expected, "Expected: %r, got %r in %s" % (expected, got, msg_loc))
    def testImplVRLittleEndianUndefLength(self):
        """Raw read: Impl VR Little Endian with undefined length................"""
        # (7fe0,0010), OB, 2-byte reserved, 4-byte-length (UNDEFINED)
        bytes1 = "e0 7f 10 00 ff ff ff ff"
        bytes2 = " 41 42 43 44 45 46 47 48 49 4a"  # 'content'
        bytes3 = " fe ff dd e0 00 00 00 00"          # Sequence Delimiter
        bytes = bytes1 + bytes2 + bytes3
        infile = StringIO(hex2str(bytes))
        expected = ((0x7fe0,0x10), 'OW or OB', 0xffffffffL, 'ABCDEFGHIJ', 0x8, True, True)
        de_gen = data_element_generator(infile, is_implicit_VR=True, is_little_endian=True)
        got = de_gen.next()
        msg_loc = "in read of undefined length Implicit VR ='OB' short value)"
        self.assertEqual(got, expected, "Expected: %r, got %r in %s" % (expected, got, msg_loc))

        # Test again such that delimiter crosses default 128-byte read "chunks", etc
        for multiplier in (116, 117, 118, 120):
            multiplier = 116
            bytes2b = bytes2 + " 00"*multiplier
            bytes = bytes1 + bytes2b + bytes3
            infile = StringIO(hex2str(bytes))
            expected = len('ABCDEFGHIJ'+'\0'*multiplier)
            de_gen = data_element_generator(infile, is_implicit_VR=True, is_little_endian=True)
            got = de_gen.next()
            got_len = len(got.value)
            msg_loc = "in read of undefined length Implicit VR with 'multiplier' %d" % multiplier
            self.assertEqual(expected, got_len, "Expected value length %d, got %d in %s" % (expected, got_len, msg_loc))
            msg = "Unexpected value start with multiplier %d on Implicit VR undefined length" % multiplier
            self.assert_(got.value.startswith('ABCDEFGHIJ\0'), msg)

class RawSequenceTests(unittest.TestCase):
    # See DICOM standard PS3.5-2008 section 7.5 for sequence syntax
    def testEmptyItem(self):
        """Read sequence with a single empty item..............................."""
        # This is fix for issue 27
        bytes = (
             "08 00 32 10"    # (0008, 1032) SQ "Procedure Code Sequence"
            " 08 00 00 00"    # length 8
            " fe ff 00 e0"    # (fffe, e000) Item Tag
            " 00 00 00 00"    # length = 0
            ) + (             # --------------- end of Sequence
            " 08 00 3e 10"    # (0008, 103e) LO "Series Description"
            " 0c 00 00 00"    # length
            " 52 20 41 44 44 20 56 49 45 57 53 20"  # value
            )
        # "\x08\x00\x32\x10\x08\x00\x00\x00\xfe\xff\x00\xe0\x00\x00\x00\x00" # from issue 27, procedure code sequence (0008,1032)
        # bytes += "\x08\x00\x3e\x10\x0c\x00\x00\x00\x52\x20\x41\x44\x44\x20\x56\x49\x45\x57\x53\x20" # data element following
        
        fp = StringIO(hex2str(bytes))
        gen = data_element_generator(fp, is_implicit_VR=True, is_little_endian=True)
        raw_seq = gen.next()
        seq = convert_value("SQ", raw_seq)
        
        self.assert_(isinstance(seq, Sequence), "Did not get Sequence, got type '%s'" % type(seq))
        self.assert_(len(seq)==1, "Expected Sequence with single (empty) item, got %d item(s)" % len(seq))
        self.assert_(len(seq[0])==0, "Expected the sequence item (dataset) to be empty")
        elem2 = gen.next()
        self.assertEqual(elem2.tag, 0x0008103e, "Expected a data element after empty sequence item")
    
    def testImplVRLittleEndian_ExplicitLengthSeq(self):
        """Raw read: ImplVR Little Endian SQ with explicit lengths.............."""
        # Create a fictional sequence with bytes directly,
        #    similar to PS 3.5-2008 Table 7.5-1 p42
        bytes = (
            "0a 30 B0 00"    # (300a, 00b0) Beam Sequence
            " 40 00 00 00"    # length
                " fe ff 00 e0"    # (fffe, e000) Item Tag
                " 18 00 00 00"    # Item (dataset) Length
                " 0a 30 c0 00"    # (300A, 00C0) Beam Number
                " 02 00 00 00"    # length
                " 31 20"          # value '1 '
                " 0a 30 c2 00"    # (300A, 00C2) Beam Name
                " 06 00 00 00"    # length
                " 42 65 61 6d 20 31" # value 'Beam 1'
                # -------------
                " fe ff 00 e0"    # (fffe, e000) Item Tag
                " 18 00 00 00"    # Item (dataset) Length
                " 0a 30 c0 00"    # (300A, 00C0) Beam Number
                " 02 00 00 00"    # length
                " 32 20"          # value '2 '
                " 0a 30 c2 00"    # (300A, 00C2) Beam Name
                " 06 00 00 00"    # length
                " 42 65 61 6d 20 32" # value 'Beam 2'                
                )
                
        infile = StringIO(hex2str(bytes))
        de_gen = data_element_generator(infile, is_implicit_VR=True, is_little_endian=True)
        raw_seq = de_gen.next()
        seq = convert_value("SQ", raw_seq)

        # The sequence is parsed, but only into raw data elements. 
        # They will be converted when asked for. Check some:
        got = seq[0].BeamNumber
        self.assert_(got == 1, "Expected Beam Number 1, got %r" % got)
        got = seq[1].BeamName
        self.assert_(got == 'Beam 2', "Expected Beam Name 'Beam 2', got %s" % got)
        
    def testImplVRBigEndian_ExplicitLengthSeq(self):
        """Raw read: ImplVR BigEndian SQ with explicit lengths.................."""
        # Create a fictional sequence with bytes directly,
        #    similar to PS 3.5-2008 Table 7.5-1 p42
        bytes = (
            "30 0a 00 B0"    # (300a, 00b0) Beam Sequence
            " 00 00 00 40"    # length
                " ff fe e0 00"    # (fffe, e000) Item Tag
                " 00 00 00 18"    # Item (dataset) Length
                " 30 0a 00 c0"    # (300A, 00C0) Beam Number
                " 00 00 00 02"    # length
                " 31 20"          # value '1 '
                " 30 0a 00 c2"    # (300A, 00C2) Beam Name
                " 00 00 00 06"    # length
                " 42 65 61 6d 20 31" # value 'Beam 1'
                # -------------
                " ff fe e0 00"    # (fffe, e000) Item Tag
                " 00 00 00 18"    # Item (dataset) Length
                " 30 0a 00 c0"    # (300A, 00C0) Beam Number
                " 00 00 00 02"    # length
                " 32 20"          # value '2 '
                " 30 0a 00 c2"    # (300A, 00C2) Beam Name
                " 00 00 00 06"    # length
                " 42 65 61 6d 20 32" # value 'Beam 2'                
                )
                
        infile = StringIO(hex2str(bytes))
        de_gen = data_element_generator(infile, is_implicit_VR=True, is_little_endian=False)
        raw_seq = de_gen.next()
        seq = convert_value("SQ", raw_seq)

        # The sequence is parsed, but only into raw data elements. 
        # They will be converted when asked for. Check some:
        got = seq[0].BeamNumber
        self.assert_(got == 1, "Expected Beam Number 1, got %r" % got)
        got = seq[1].BeamName
        self.assert_(got == 'Beam 2', "Expected Beam Name 'Beam 2', got %s" % got)

    def testExplVRBigEndian_UndefinedLengthSeq(self):
        """Raw read: ExplVR BigEndian Undefined Length SQ......................."""
        # Create a fictional sequence with bytes directly,
        #    similar to PS 3.5-2008 Table 7.5-2 p42
        item1_value_bytes = "\1"*126
        item2_value_bytes = "\2"*222
        bytes = (
            "30 0a 00 B0"    # (300a, 00b0) Beam Sequence
            " 53 51"         # SQ
            " 00 00"         # reserved
            " ff ff ff ff"    # undefined length
                " ff fe e0 00"    # (fffe, e000) Item Tag
                " 00 00 00 18"    # Item (dataset) Length
                " 30 0a 00 c0"    # (300A, 00C0) Beam Number
                " 49 53"          # IS
                " 00 02"          # length
                " 31 20"          # value '1 '
                " 30 0a 00 c2"    # (300A, 00C2) Beam Name
                " 4c 4F"          # LO
                " 00 06"          # length
                " 42 65 61 6d 20 31" # value 'Beam 1'
                # -------------
                " ff fe e0 00"    # (fffe, e000) Item Tag
                " 00 00 00 18"    # Item (dataset) Length
                " 30 0a 00 c0"    # (300A, 00C0) Beam Number
                " 49 53"          # IS
                " 00 02"          # length
                " 32 20"          # value '2 '
                " 30 0a 00 c2"    # (300A, 00C2) Beam Name
                " 4C 4F"          # LO
                " 00 06"          # length
                " 42 65 61 6d 20 32" # value 'Beam 2'   
            " ff fe E0 dd"    # SQ delimiter
            " 00 00 00 00"    # zero length              
                )
                
        infile = StringIO(hex2str(bytes))
        de_gen = data_element_generator(infile, is_implicit_VR=False, is_little_endian=False)
        seq = de_gen.next()
        # Note seq itself is not a raw data element. 
        #     The parser does parse undefined length SQ

        # The sequence is parsed, but only into raw data elements. 
        # They will be converted when asked for. Check some:
        got = seq[0].BeamNumber
        self.assert_(got == 1, "Expected Beam Number 1, got %r" % got)
        got = seq[1].BeamName
        self.assert_(got == 'Beam 2', "Expected Beam Name 'Beam 2', got %s" % got)
        
if __name__ == "__main__":
    # import dicom
    # dicom.debug()
    unittest.main()



# dicom_series.py
"""
By calling the function read_files with a directory name or list
of files as an argument, a list of DicomSeries instances can be
obtained. A DicomSeries object has some attributes that give 
information about the serie (such as shape, sampling, suid) and
has an info attribute, which is a dicom.DataSet instance containing
information about the first dicom file in the serie. The data can
be obtained using the get_pixel_array() method, which produces a
3D numpy array if there a multiple files in the serie. 

This module can deal with gated data, in which case a DicomSeries
instance is created for each 3D volume.

"""
#
# Copyright (c) 2010 Almar Klein
# This file is released under the pydicom license.
#    See the file license.txt included with the pydicom distribution, also
#    available at http://pydicom.googlecode.com
#


# I (Almar) performed some test to loading a series of data
# in two different ways: loading all data, and deferring loading
# the data. Both ways seem equally fast on my system. I have to
# note that results can differ quite a lot depending on the system,
# but still I think this suggests that deferred reading is in 
# general not slower. I think deferred loading of the pixel data
# can be advantageous because maybe not all data of all series
# is needed. Also it simply saves memory, because the data is
# removed from the Dataset instances.
# In the few result below, cold means reading for the first time,
# warm means reading 2nd/3d/etc time.
# - Full loading of data, cold: 9 sec
# - Full loading of data, warm: 3 sec
# - Deferred loading of data, cold: 9 sec
# - Deferred loading of data, warm: 3 sec


import os, sys, time, gc
import dicom
from dicom.sequence import Sequence

# Try importing numpy
try:
    import numpy as np
    have_numpy = True
except Exception:
    np = None
    have_numpy = False



## Helper functions and classes



class ProgressBar:
    """ To print progress to the screen. 
    """
    def __init__(self, char='-', length=20):
        self.char = char
        self.length = length
        self.progress = 0.0
        self.nbits = 0
        self.what = ''
    
    def Start(self, what=''):
        """ Start(what='')
        Start the progress bar, displaying the given text first.
        Make sure not to print anything untill after calling 
        Finish(). Messages can be printed while displaying 
        progess by using printMessage().
        """ 
        self.what = what
        self.progress = 0.0 
        self.nbits = 0
        sys.stdout.write(what+" [")
    
    def Stop(self, message=""):
        """ Stop the progress bar where it is now. 
        Optionally print a message behind it."""        
        delta = int(self.length - self.nbits)
        sys.stdout.write( " "*delta + "] " + message + "\n")
    
    def Finish(self, message=""):
        """ Finish the progress bar, setting it to 100% if it 
        was not already. Optionally print a message behind the bar.
        """
        delta = int(self.length-self.nbits)
        sys.stdout.write( self.char*delta + "] " + message + "\n")
    
    def Update(self, newProgress):
        """ Update progress. Progress is given as a number 
        between 0 and 1.
        """
        self.progress = newProgress
        required = self.length*(newProgress)
        delta = int(required-self.nbits)
        if delta>0:
            sys.stdout.write(self.char*delta)
            self.nbits += delta
    
    def PrintMessage(self, message):
        """ Print a message (for example a warning).
        The message is printed behind the progress bar, 
        and a new bar is started.
        """
        self.Stop(message)
        self.Start(self.what)


def _dummyProgressCallback(progress):
    """ A callback to indicate progress that does nothing. """
    pass

_progressBar = ProgressBar()
def _progressCallback(progress):
    """ The default callback for displaying progress. """
    if isinstance(progress, basestring):
        _progressBar.Start(progress)
        _progressBar._t0 = time.time()
    elif progress is None:
        dt = time.time() - _progressBar._t0
        _progressBar.Finish('%2.2f seconds' % dt)
    else:
        _progressBar.Update(progress)




def _listFiles(files, path):
    """List all files in the directory, recursively. """
    
    for item in os.listdir(path):
        item = os.path.join(path, item)
        if os.path.isdir( item ):
            _listFiles(files, item)
        else:
            files.append(item)


def _splitSerieIfRequired(serie, series):
    """ _splitSerieIfRequired(serie, series)
    Split the serie in multiple series if this is required.
    The choice is based on examing the image position relative to
    the previous image. If it differs too much, it is assumed
    that there is a new dataset. This can happen for example in 
    unspitted gated CT data.
    """
    
    # Sort the original list and get local name
    serie._sort()
    L = serie._datasets
    
    # Init previous slice 
    ds1 = L[0]
    
    # Check whether we can do this
    if not "ImagePositionPatient" in ds1:
        return
    
    # Initialize a list of new lists
    L2 = [[ds1]]
    
    # Init slice distance estimate
    distance = 0
    
    for index in range(1,len(L)):
        
        # Get current slice
        ds2 = L[index]
        
        # Get positions
        pos1 = ds1.ImagePositionPatient[2] 
        pos2 = ds2.ImagePositionPatient[2] 
        
        # Get distances
        newDist = abs(pos1 - pos2)
        #deltaDist = abs(firstPos-pos2)
        
        # If the distance deviates more than 2x from what we've seen, 
        # we can agree it's a new dataset.
        if distance and newDist > 2.1*distance:
            L2.append([])
            distance = 0            
        else:
            # Test missing file
            if distance and newDist > 1.5*distance:
                print 'Warning: missing file after "%s"' % ds1.filename
            distance = newDist
        
        # Add to last list
        L2[-1].append( ds2 )
        
        # Store previous
        ds1 = ds2
    
    
    # Split if we should
    if len(L2) > 1:
        
        # At what position are we now?
        i = series.index(serie)
        
        # Create new series            
        series2insert = []
        for L in L2:
            newSerie = DicomSeries(serie.suid, serie._showProgress)
            newSerie._datasets = Sequence(L)
            series2insert.append(newSerie)
        
        # Insert series and remove self
        for newSerie in reversed(series2insert):
            series.insert(i, newSerie)
        series.remove(serie)


pixelDataTag = dicom.tag.Tag(0x7fe0, 0x0010)

def _getPixelDataFromDataset(ds):
    """ Get the pixel data from the given dataset. If the data
    was deferred, make it deferred again, so that memory is 
    preserved. Also applies RescaleSlope and RescaleIntercept
    if available. """
    
    # Get original element
    el = dict.__getitem__(ds, pixelDataTag)    
    
    # Get data
    data = ds.PixelArray
    
    # Remove data (mark as deferred)            
    dict.__setitem__(ds, pixelDataTag, el)
    del ds._PixelArray
    
    # Obtain slope and offset
    slope = 1
    offset = 0
    needFloats = False
    needApplySlopeOffset = False
    if 'RescaleSlope' in ds:
        needApplySlopeOffset = True
        slope = ds.RescaleSlope
    if 'RescaleIntercept' in ds:
        needApplySlopeOffset = True
        offset = ds.RescaleIntercept
    if int(slope)!= slope or int(offset) != offset:
        needFloats = True
    if not needFloats:
        slope, offset = int(slope), int(offset)
    
    # Apply slope and offset
    if needApplySlopeOffset:
        
        # Maybe we need to change the datatype?
        if data.dtype in [np.float32, np.float64]:
            pass
        elif needFloats:
            data = data.astype(np.float32)
        else:
            # Determine required range
            minReq, maxReq = data.min(), data.max()
            minReq = min([minReq, minReq*slope+offset, maxReq*slope+offset])
            maxReq = max([maxReq, minReq*slope+offset, maxReq*slope+offset])
            
            # Determine required datatype from that
            dtype = None
            if minReq<0:
                # Signed integer type
                maxReq = max([-minReq, maxReq])
                if maxReq < 2**7:
                    dtype = np.int8
                elif maxReq < 2**15:
                    dtype = np.int16
                elif maxReq < 2**31:
                    dtype = np.int32
                else:
                    dtype = np.float32
            else:
                # Unsigned integer type
                if maxReq < 2**8:
                    dtype = np.int8
                elif maxReq < 2**16:
                    dtype = np.int16
                elif maxReq < 2**32:
                    dtype = np.int32
                else:
                    dtype = np.float32
            
            # Change datatype
            if dtype != data.dtype:
                data = data.astype(dtype)
        
        # Apply slope and offset
        data *= slope
        data += offset
    
    # Done
    return data


## The public functions and classes

def read_files(path, showProgress=False, readPixelData=False):
    """ read_files(path, showProgress=False, readPixelData=False)
    
    Reads dicom files and returns a list of DicomSeries objects, which 
    contain information about the data, and can be used to load the 
    image or volume data.
    
    The parameter "path" can also be a list of files or directories.
    
    If the callable "showProgress" is given, it is called with a single
    argument to indicate the progress. The argument is a string when a
    progress is started (indicating what is processed). A float indicates
    progress updates. The paremeter is None when the progress is finished. 
    When "showProgress" is True, a default callback is used that writes 
    to stdout. By default, no progress is shown.
    
    if readPixelData is True, the pixel data of all series is read. By
    default the loading of pixeldata is deferred until it is requested
    using the DicomSeries.get_pixel_array() method. In general, both 
    methods should be equally fast.
    """
    
    # Init list of files
    files = []
    
    # Obtain data from the given path
    if isinstance(path, basestring):
        # Make dir nice
        basedir = os.path.abspath(path)
        # Check whether it exists
        if not os.path.isdir(basedir):
            raise ValueError('The given path is not a valid directory.')
        # Find files recursively
        _listFiles(files, basedir)
        
    elif isinstance(path, (tuple, list)):
        # Iterate over all elements, which can be files or directories
        for p in path:
            if os.path.isdir(p):
                _listFiles(files, os.path.abspath(p))
            elif os.path.isfile(p):
                files.append(p)
            else:
                print "Warning, the path '%s' is not valid." % p
    else:
        raise ValueError('The path argument must be a string or list.')
    
    
    # Set default progress callback?
    if showProgress is True:
        showProgress = _progressCallback
    if not hasattr(showProgress, '__call__'):
        showProgress = _dummyProgressCallback
    
    
    # Set defer size
    deferSize = 16383 # 128**2-1
    if readPixelData:
        deferSize = None
    
    # Gather file data and put in DicomSeries    
    series = {}
    count = 0    
    showProgress('Loading series information:')
    for filename in files:
        
        # Skip DICOMDIR files
        if filename.count("DICOMDIR"):
            continue
        
        # Try loading dicom ...
        try:
            dcm = dicom.read_file( filename, deferSize )
        except dicom.filereader.InvalidDicomError:
            continue # skip non-dicom file
        except Exception, why:
            if showProgress is _progressCallback:
                _progressBar.PrintMessage(str(why))
            else:
                print 'Warning:', why
            continue
        
        # Get SUID and register the file with an existing or new series object
        try:
            suid = dcm.SeriesInstanceUID
        except AttributeError:
            continue # some other kind of dicom file
        if suid not in series:
            series[suid] = DicomSeries(suid, showProgress)
        series[suid]._append(dcm)
        
        # Show progress (note that we always start with a 0.0)        
        showProgress( float(count) / len(files) )
        count += 1
    
    # Finish progress
    showProgress( None )
    
    # Make a list and sort, so that the order is deterministic
    series = series.values()
    series.sort(key=lambda x:x.suid)
    
    # Split series if necessary
    for serie in reversed([serie for serie in series]):
        _splitSerieIfRequired(serie, series)
    
    # Finish all series
    showProgress('Analysing series')
    series_ = []
    for i in range(len(series)):
        try:
            series[i]._finish()
            series_.append(series[i])
        except Exception:
            pass # Skip serie (probably report-like file without pixels)
        showProgress(float(i+1)/len(series))
    showProgress(None)
    
    return series_


class DicomSeries(object):
    """ DicomSeries
    This class represents a serie of dicom files that belong together. 
    If these are multiple files, they represent the slices of a volume 
    (like for CT or MRI). The actual volume can be obtained using loadData(). 
    Information about the data can be obtained using the info attribute.
    """
    
    # To create a DicomSeries object, start by making an instance and
    # append files using the "_append" method. When all files are
    # added, call "_sort" to sort the files, and then "_finish" to evaluate
    # the data, perform some checks, and set the shape and sampling
    # attributes of the instance.
    
    def __init__(self, suid, showProgress):        
        # Init dataset list and the callback
        self._datasets = Sequence()        
        self._showProgress = showProgress
        
        # Init props
        self._suid = suid
        self._info = None
        self._shape = None
        self._sampling = None
    
    
    @property
    def suid(self):
        """ The Series Instance UID. """ 
        return self._suid
    
    
    @property
    def shape(self):
        """ The shape of the data (nz, ny, nx). 
        If None, the serie contains a single dicom file. """
        return self._shape
    
    
    @property
    def sampling(self):
        """ The sampling (voxel distances) of the data (dz, dy, dx). 
        If None, the serie contains a single dicom file. """
        return self._sampling
    
    
    @property
    def info(self):
        """ A DataSet instance containing the information as present in the
        first dicomfile of this serie. """
        return self._info
    
    
    @property
    def description(self):
        """ A description of the dicom series. Used fields are
        PatientName, shape of the data, SeriesDescription, 
        and ImageComments.
        """
        
        info = self.info
        
        # If no info available, return simple description
        if info is None:
            return "DicomSeries containing %i images" % len(self._datasets)
        
        fields = []
        
        # Give patient name
        if 'PatientName' in info:
            fields.append(""+info.PatientName)
        
        # Also add dimensions
        if self.shape:
            tmp = [str(d) for d in self.shape]        
            fields.append( 'x'.join(tmp) )
        
        # Try adding more fields        
        if 'SeriesDescription' in info:
            fields.append("'"+info.SeriesDescription+"'")
        if 'ImageComments' in info:
            fields.append("'"+info.ImageComments+"'")
        
        # Combine
        return ' '.join(fields)
    
    
    def __repr__(self):
        adr = hex(id(self)).upper()
        return "<DicomSeries with %i images at %s>" % (len(self._datasets), adr)
    
    
    def get_pixel_array(self):
        """ get_pixel_array()
        
        Get (load) the data that this DicomSeries represents, and return
        it as a numpy array. If this serie contains multiple images, the
        resulting array is 3D, otherwise it's 2D.
        
        If RescaleSlope and RescaleIntercept are present in the dicom info,
        the data is rescaled using these parameters. The data type is chosen
        depending on the range of the (rescaled) data.
        
        """
        
        # Can we do this?
        if not have_numpy:
            msg = "The Numpy package is required to use get_pixel_array.\n"
            raise ImportError(msg)
        
        # It's easy if no file or if just a single file
        if len(self._datasets)==0:
            raise ValueError('Serie does not contain any files.')
        elif len(self._datasets)==1:
            ds = self._datasets[0]
            slice = _getPixelDataFromDataset( ds )
            return slice
        
        # Check info
        if self.info is None:
            raise RuntimeError("Cannot return volume if serie is not finished.")
        
        # Set callback to update progress
        showProgress = self._showProgress
        
        # Init data (using what the dicom packaged produces as a reference)
        ds = self._datasets[0]
        slice = _getPixelDataFromDataset( ds )
        #vol = Aarray(self.shape, self.sampling, fill=0, dtype=slice.dtype)
        vol = np.zeros(self.shape, dtype=slice.dtype)
        vol[0] = slice
        
        # Fill volume
        showProgress('Loading data:')
        ll = self.shape[0]
        for z in range(1,ll):            
            ds = self._datasets[z]            
            vol[z] = _getPixelDataFromDataset(ds)
            showProgress(float(z)/ll)
        
        # Finish
        showProgress(None)
        
        # Done
        gc.collect()
        return vol
    
    
    def _append(self, dcm):
        """ _append(dcm)
        Append a dicomfile (as a dicom.dataset.FileDataset) to the series.
        """
        self._datasets.append(dcm)
    
    
    def _sort(self):
        """ sort()
        Sort the datasets by instance number. 
        """
        self._datasets.sort(key=lambda k: k.InstanceNumber)
    
    
    def _finish(self):
        """ _finish()
        
        Evaluate the series of dicom files. Together they should make up
        a volumetric dataset. This means the files should meet certain 
        conditions. Also some additional information has to be calculated,
        such as the distance between the slices. This method sets the
        attributes for "shape", "sampling" and "info".
        
        This method checks:
          * that there are no missing files
          * that the dimensions of all images match
          * that the pixel spacing of all images match
        
        """
        
        # The datasets list should be sorted by instance number
        L = self._datasets
        if len(L)==0:
            return
        elif len(L) < 2:
            # Set attributes
            ds = self._datasets[0]
            self._info = self._datasets[0]
            self._shape = [ds.Rows, ds.Columns]
            self._sampling = [ds.PixelSpacing[0], ds.PixelSpacing[1]]
            return
        
        # Get previous
        ds1 = L[0]
        
        # Init measures to calculate average of
        distance_sum = 0.0
        
        # Init measures to check (these are in 2D)
        dimensions = ds1.Rows, ds1.Columns
        sampling = ds1.PixelSpacing[0], ds1.PixelSpacing[1] # row, column
        
        
        for index in range(len(L)):
            # The first round ds1 and ds2 will be the same, for the
            # distance calculation this does not matter
            
            # Get current
            ds2 = L[index]
            
            # Get positions
            pos1 = ds1.ImagePositionPatient[2]
            pos2 = ds2.ImagePositionPatient[2] 
            
            # Update distance_sum to calculate distance later
            distance_sum += abs(pos1 - pos2)
            
            # Test measures
            dimensions2 = ds2.Rows, ds2.Columns
            sampling2 = ds2.PixelSpacing[0], ds2.PixelSpacing[1]
            if dimensions != dimensions2:
                # We cannot produce a volume if the dimensions match
                raise ValueError('Dimensions of slices does not match.')
            if sampling != sampling2:
                # We can still produce a volume, but we should notify the user
                msg = 'Warning: sampling does not match.'
                if self._showProgress is _progressCallback:                    
                    _progressBar.PrintMessage(msg)
                else:
                    print msg
            # Store previous
            ds1 = ds2
        
        
        # Create new dataset by making a deep copy of the first
        info = dicom.dataset.Dataset()
        firstDs = self._datasets[0]
        for key in firstDs.keys():
            if key != (0x7fe0, 0x0010):
                el = firstDs[key]
                info.add_new(el.tag, el.VR, el.value)
        
        # Finish calculating average distance 
        # (Note that there are len(L)-1 distances)
        distance_mean = distance_sum / (len(L)-1)
        
        # Store information that is specific for the serie
        self._shape = [len(L), ds2.Rows, ds2.Columns]
        self._sampling = [distance_mean, ds2.PixelSpacing[0], ds2.PixelSpacing[1]]
        
        # Store
        self._info = info


if __name__ == '__main__':
    import sys

    if len(sys.argv) != 2:
        print "Expected a single argument: a directory with dicom files in it"
    else:
        adir = sys.argv[1]
        t0 = time.time()
        all_series = read_files(adir, None, False)
        print "Summary of each series:"
        for series in all_series:
            print series.description



# ListBeams.py
"""Given an RTPLAN DICOM file, list basic info for the beams in it
"""
# Copyright (c) 2008-2011 Darcy Mason
# This file is part of pydicom, relased under an MIT license.
#    See the file license.txt included with this distribution, also
#    available at http://pydicom.googlecode.com

import dicom

usage = """python ListBeams.py rtplan.dcm"""

def ListBeams(plan_dataset):
    """Return a string summarizing the RTPLAN beam information in the dataset"""
    lines = ["%13s %8s %8s %8s" % ("Beam name", "Number", "Gantry", "SSD (cm)")]
    for beam in plan_dataset.BeamSequence:
        cp0 = beam.ControlPointSequence[0]
        SSD = float(cp0.SourcetoSurfaceDistance / 10)
        lines.append("%13s %8s %8.1f %8.1f" % (beam.BeamName, str(beam.BeamNumber),
                                      cp0.GantryAngle, SSD))
    return "\n".join(lines)

if __name__ == "__main__":
    import sys
    if len(sys.argv) != 2:
        print usage
        sys.exit(-1)

    rtplan = dicom.read_file(sys.argv[1])
    print ListBeams(rtplan)


# test_dataset.py
"""unittest cases for dicom.dataset module"""
# Copyright (c) 2008-2012 Darcy Mason
# This file is part of pydicom, released under a modified MIT license.
#    See the file license.txt included with this distribution, also
#    available at http://pydicom.googlecode.com

import unittest
from dicom.dataset import Dataset, have_numpy, PropertyError
from dicom.dataelem import DataElement, RawDataElement
from dicom.tag import Tag
from dicom.sequence import Sequence

class DatasetTests(unittest.TestCase):
    def failUnlessRaises(self, excClass, callableObj, *args, **kwargs):
        """Redefine unittest Exception test to return the exception object"""
        # from http://stackoverflow.com/questions/88325/how-do-i-unit-test-an-init-method-of-a-python-class-with-assertraises
        try:
            callableObj(*args, **kwargs)
        except excClass, excObj:
            return excObj # Actually return the exception object
        else:
            if hasattr(excClass,'__name__'): excName = excClass.__name__
            else: excName = str(excClass)
            raise self.failureException, "%s not raised" % excName

    def failUnlessExceptionArgs(self, start_args, excClass, callableObj):
        """Check the expected args were returned from an exception
        start_args -- a string with the start of the expected message
        """
        # based on same link as failUnlessRaises override above
        excObj = self.failUnlessRaises(excClass, callableObj)
        msg = "\nExpected Exception message:\n" + start_args + "\nGot:\n" + excObj[0]
        self.assertTrue(excObj[0].startswith(start_args), msg)

    def testAttributeErrorInProperty(self):
        """Dataset: AttributeError in property raises actual error message..."""
        # This comes from bug fix for issue 42
        # First, fake enough to try the pixel_array property
        ds = Dataset()
        ds.file_meta = Dataset()
        ds.PixelData = 'xyzlmnop'
        attribute_error_msg = "AttributeError in pixel_array property: " + \
                           "Dataset does not have attribute 'TransferSyntaxUID'"
        self.failUnlessExceptionArgs(attribute_error_msg,
                        PropertyError, ds._get_pixel_array)
        
        
    def dummy_dataset(self):
        # This dataset is used by many of the tests
        ds = Dataset()
        ds.add_new((0x300a, 0x00b2), "SH", "unit001") # TreatmentMachineName
        return ds
        
    def testSetNewDataElementByName(self):
        """Dataset: set new data_element by name............................."""
        ds = Dataset()
        ds.TreatmentMachineName = "unit #1"
        data_element = ds[0x300a, 0x00b2]
        self.assertEqual(data_element.value, "unit #1", "Unable to set data_element by name")
        self.assertEqual(data_element.VR, "SH", "data_element not the expected VR")
    def testSetExistingDataElementByName(self):
        """Dataset: set existing data_element by name........................"""
        ds = self.dummy_dataset()
        ds.TreatmentMachineName = "unit999" # change existing value
        self.assertEqual(ds[0x300a, 0x00b2].value, "unit999")
    def testSetNonDicom(self):
        """Dataset: can set class instance property (non-dicom).............."""
        ds = Dataset()
        ds.SomeVariableName = 42
        has_it = hasattr(ds, 'SomeVariableName')
        self.assert_(has_it, "Variable did not get created")
        if has_it:
            self.assertEqual(ds.SomeVariableName, 42, "There, but wrong value")
    def testMembership(self):
        """Dataset: can test if item present by 'if <name> in dataset'......."""
        ds = self.dummy_dataset()
        self.assert_('TreatmentMachineName' in ds, "membership test failed")
        self.assert_(not 'Dummyname' in ds, "non-member tested as member")
    def testContains(self):
        """Dataset: can test if item present by 'if <tag> in dataset'........"""
        ds = self.dummy_dataset()
        self.assert_((0x300a, 0xb2) in ds, "membership test failed")
        self.assert_([0x300a, 0xb2] in ds, "membership test failed when list used")
        self.assert_(0x300a00b2 in ds, "membership test failed")
        self.assert_(not (0x10,0x5f) in ds, "non-member tested as member")        
    def testGetExists1(self):
        """Dataset: dataset.get() returns an existing item by name..........."""
        ds = self.dummy_dataset()
        unit = ds.get('TreatmentMachineName', None)
        self.assertEqual(unit, 'unit001', "dataset.get() did not return existing member by name")
    def testGetExists2(self):
        """Dataset: dataset.get() returns an existing item by long tag......."""
        ds = self.dummy_dataset()    
        unit = ds.get(0x300A00B2, None).value
        self.assertEqual(unit, 'unit001', "dataset.get() did not return existing member by long tag")
    def testGetExists3(self):
        """Dataset: dataset.get() returns an existing item by tuple tag......"""
        ds = self.dummy_dataset()        
        unit = ds.get((0x300A, 0x00B2), None).value
        self.assertEqual(unit, 'unit001', "dataset.get() did not return existing member by tuple tag")
    def testGetExists4(self):
        """Dataset: dataset.get() returns an existing item by Tag............"""
        ds = self.dummy_dataset()        
        unit = ds.get(Tag(0x300A00B2), None).value
        self.assertEqual(unit, 'unit001', "dataset.get() did not return existing member by tuple tag")     
    def testGetDefault1(self):
        """Dataset: dataset.get() returns default for non-existing name ....."""
        ds = self.dummy_dataset()
        not_there = ds.get('NotAMember', "not-there")
        self.assertEqual(not_there, "not-there",
                    "dataset.get() did not return default value for non-member by name")
    def testGetDefault2(self):
        """Dataset: dataset.get() returns default for non-existing tuple tag."""
        ds = self.dummy_dataset()                    
        not_there = ds.get((0x9999, 0x9999), "not-there")
        self.assertEqual(not_there, "not-there",
                    "dataset.get() did not return default value for non-member by tuple tag")
    def testGetDefault3(self):
        """Dataset: dataset.get() returns default for non-existing long tag.."""
        ds = self.dummy_dataset()                    
        not_there = ds.get(0x99999999, "not-there")
        self.assertEqual(not_there, "not-there",
                    "dataset.get() did not return default value for non-member by long tag")
    def testGetDefault4(self):
        """Dataset: dataset.get() returns default for non-existing Tag......."""
        ds = self.dummy_dataset()                    
        not_there = ds.get(Tag(0x99999999), "not-there")
        self.assertEqual(not_there, "not-there",
                    "dataset.get() did not return default value for non-member by Tag")
    def testGetFromRaw(self):
        """Dataset: get(tag) returns same object as ds[tag] for raw element.."""
        # This came from issue 88, where get(tag#) returned a RawDataElement, 
        #     while get(name) converted to a true DataElement
        test_tag = 0x100010
        test_elem = RawDataElement(Tag(test_tag), 'PN', 4, 'test', 0, True, True) 
        ds = Dataset({Tag(test_tag): test_elem})
        by_get = ds.get(test_tag)
        by_item = ds[test_tag]
        
        # self.assertEqual(type(elem_get), type(name_get), "Dataset.get() returned different type for name vs tag access")
        msg = "Dataset.get() returned different objects for ds.get(tag) and ds[tag]:\nBy get():%r\nBy ds[tag]:%r\n"
        self.assertEqual(by_get, by_item, msg % (by_get, by_item))

    def test__setitem__(self):
        """Dataset: if set an item, it must be a DataElement instance........"""
        def callSet():
            ds[0x300a, 0xb2]="unit1" # common error - set data_element instead of data_element.value
            
        ds = Dataset()
        self.assertRaises(TypeError, callSet)
    def test_matching_tags(self):
        """Dataset: key and data_element.tag mismatch raises ValueError......"""
        def set_wrong_tag():
            ds[0x10,0x10] = data_element
        ds = Dataset()
        data_element = DataElement((0x300a, 0x00b2), "SH", "unit001")
        self.assertRaises(ValueError, set_wrong_tag)
    def test_NamedMemberUpdated(self):
        """Dataset: if set data_element by tag, name also reflects change...."""
        ds = self.dummy_dataset()
        ds[0x300a,0xb2].value = "moon_unit"
        self.assertEqual(ds.TreatmentMachineName, 'moon_unit', "Member not updated")
    def testUpdate(self):
        """Dataset: update() method works with tag or name..................."""
        ds = self.dummy_dataset()
        pat_data_element = DataElement((0x10,0x12), 'PN', 'Johnny')
        ds.update({'PatientName': 'John', (0x10,0x12): pat_data_element})
        self.assertEqual(ds[0x10,0x10].value, 'John', "named data_element not set")
        self.assertEqual(ds[0x10,0x12].value, 'Johnny', "set by tag failed")
    def testDir(self):
        """Dataset: dir() returns sorted list of named data_elements........."""
        ds = self.dummy_dataset()
        ds.PatientName = "name"
        ds.PatientID = "id"
        ds.NonDicomVariable = "junk"
        ds.add_new((0x18,0x1151), "IS", 150) # X-ray Tube Current
        ds.add_new((0x1111, 0x123), "DS", "42.0") # private tag - no name in dir()
        expected = ['PatientID', 'PatientName', 'TreatmentMachineName', 'XRayTubeCurrent']
        self.assertEqual(ds.dir(), expected, "dir() returned %s, expected %s" % (str(ds.dir()), str(expected)))
    def testDeleteDicomAttr(self):
        """Dataset: delete DICOM attribute by name..........................."""
        def testAttribute():
            ds.TreatmentMachineName
        
        ds = self.dummy_dataset()
        del ds.TreatmentMachineName
        self.assertRaises(AttributeError, testAttribute)        
    def testDeleteOtherAttr(self):
        """Dataset: delete non-DICOM attribute by name......................."""
        ds = self.dummy_dataset()
        ds.meaningoflife = 42
        del ds.meaningoflife
    def testDeleteDicomAttrWeDontHave(self):
        """Dataset: try delete of missing DICOM attribute...................."""
        def try_delete():
            del ds.PatientName
        ds = self.dummy_dataset()
        self.assertRaises(AttributeError, try_delete)

class DatasetElementsTests(unittest.TestCase):
    """Test valid assignments of data elements"""
    def setUp(self):
        self.ds = Dataset()
        self.sub_ds1 = Dataset()
        self.sub_ds2 = Dataset()
    def testSequenceAssignment(self):
        """Assignment to SQ works only if valid Sequence assigned......"""
        def try_non_Sequence():
            self.ds.ConceptCodeSequence = [1,2,3]
        msg = "Assigning a non-sequence to an SQ data element did not raise error"
        self.assertRaises(TypeError, try_non_Sequence)  
        # check also that assigning proper sequence *does* work
        self.ds.ConceptCodeSequence = [self.sub_ds1, self.sub_ds2]
        self.assert_(isinstance(self.ds.ConceptCodeSequence, Sequence),
                "Sequence assignment did not result in Sequence type")   
    
if __name__ == "__main__":
    unittest.main()
